package main;
import java.text.Collator;
import java.util.ArrayList;
import java.util.List;
import java.util.Set;
import java.util.TreeSet;


/**
 * Class with base methods for implementing a data combination criteria.
 * 
 * @author ernestocid
 *
 */
public abstract class Criteria {
	
	
	private List<List<String>> parametersInputValues;
	
	
	/**
	 * The constructor filters the empty lists that might be added 
	 * on the list of values.
	 * 
	 * @param parametersInputValues: values for each parameter
	 */
	public Criteria(List<List<String>> parametersInputValues) {
		List<List<String>> filteredList = new ArrayList<List<String>>();
		
		// Filtering empty lists
		for(List<String> parameterInputValue : parametersInputValues) {
			if(!parameterInputValue.isEmpty()) {
				filteredList.add(parameterInputValue);
			}
		}
		
		this.parametersInputValues = filteredList;
	}

	
	
	public List<List<String>> getParametersInputValues() {
		return parametersInputValues;
	}

	
	
	public void setParametersInputValues(List<List<String>> parametersInputValues) {
		this.parametersInputValues = parametersInputValues;
	}
	
	
	
	/**
	 * Returns each test combination as a String.
	 * 
	 * @return combinationsAsStrings: set of combinations in the form of Strings
	 */
	public Set<String> getCombinationsAsStrings() {
		Set<List<String>> combinations = getCombinations();
		
		Set<String> combinationsAsStrings = new TreeSet<String>(Collator.getInstance());
		
		for (List<String> combination : combinations) {
			StringBuffer string = new StringBuffer("");
			int valueCount = 1;
			for (String value : combination) {
				if(valueCount < combination.size()) {
					string.append(value + " & ");
				} else {
					string.append(value);
				}
				valueCount++;
			}
			
			combinationsAsStrings.add(string.toString());
			
		}
		
		return combinationsAsStrings;
	}
	
	
	
	public abstract Set<List<String>> getCombinations();
	
}
