package main;

import java.util.ArrayList;
import java.util.List;

public class PairCollection {

	
	private List<TestPair> pairs = new ArrayList<TestPair>();

	
	public PairCollection(List<TestPair> pairs) {
		this.pairs = pairs;
	}

	
	
	public PairCollection(List<String> inputParameterValues, List<List<String>> inputValueLists, int paramIndex) {
	
		this.pairs = new ArrayList<TestPair>();
		
		for(String value : inputParameterValues) {
			int paramBIndex = 0;
			for(List<String> list : inputValueLists) {
				for (String value2 : list) {
					pairs.add(new TestPair(paramIndex, paramBIndex, value, value2));
				}
				paramBIndex++;
			}
		}
		
	}

	
	
	public List<TestPair> getPairs() {
		return pairs;
	}
	
	
	
	public List<List<String>> toList() {
	
		List<List<String>> valuesList = new ArrayList<List<String>>();
		
		for(TestPair pair : pairs) {
			List<String> pairList = new ArrayList<String>();
			pairList.add(pair.getValueA());
			pairList.add(pair.getValueB());
			valuesList.add(pairList);
		}
		
		return valuesList;
		
	}

	
	
	public void removePairsCoveredBy(List<String> combination) {
		
		List<TestPair> newPairs = new ArrayList<TestPair>(pairs);
		
		for (TestPair testPair : newPairs) {
			boolean firstValueFromPairEqualsValueFromCombination = combination.get(testPair.getValueAIndex()).equals(testPair.getValueA());
			boolean secondValueFromPairEqualsValueFromCombination = combination.get(testPair.getValueBIndex()).equals(testPair.getValueB());
			
			if(firstValueFromPairEqualsValueFromCombination && secondValueFromPairEqualsValueFromCombination) {
				pairs.remove(testPair);
			}
		}
		
	}

	
	
	public List<String> getCombinationsThatCoversMostPairs(List<String> combination, List<String> inputValuesForGrowth) {
		
		List<String> selectedInputCombination = null;
		int maxCoveredCount = 0;
		int coveredCount = 0;
		
		for(String inputValue : inputValuesForGrowth) {
			List<String> combinationCandidate = new ArrayList<String>(combination);
			combinationCandidate.add(inputValue);
			
			coveredCount = calculateCoveredPairsCount(combinationCandidate);
			
			if(coveredCount > maxCoveredCount) {
				maxCoveredCount = coveredCount;
				selectedInputCombination = combinationCandidate;
			}
		}
		
		if(selectedInputCombination == null) {
			selectedInputCombination = new ArrayList<String>(combination);
			selectedInputCombination.add(inputValuesForGrowth.get(0));
		}
		
		return selectedInputCombination;
		
	}

	
	
	private int calculateCoveredPairsCount(List<String> combination) {
	
		int coveredCount = 0;
		
		for (TestPair pair : pairs) {
			if(pair.isCoveredBy(combination)) {
				coveredCount++;
			}
		}
		
		return coveredCount;
		
	}
}
